/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef LOG_ENTRY_HH_C42FFA9E17BE44D48FAB7F5FB13A9E02
#define LOG_ENTRY_HH_C42FFA9E17BE44D48FAB7F5FB13A9E02

#include "libpg/pg_rw_transaction.hh"
#include "libpg/util/strong_type.hh"

#include <boost/asio/ip/address.hpp>
#include <boost/date_time/posix_time/ptime.hpp>

#include <string>
#include <tuple>
#include <vector>

namespace Fred {
namespace LibLogger {

namespace LogEntry {

using TimeBegin = LibPg::Util::StrongType<boost::posix_time::ptime, struct TimeBeginTag_>;
using TimeEnd = LibPg::Util::StrongType<boost::posix_time::ptime, struct TimeEndTag_>;
using SourceIp = LibPg::Util::StrongType<boost::asio::ip::address, struct SourceIpTag_>;
using ResultCode = LibPg::Util::StrongType<unsigned long long, struct ResultCodeTag_>;
using ResultName = LibPg::Util::StrongType<std::string, struct ResultNameTag_>;

using SessionId = LibPg::Util::StrongType<unsigned long long, struct SessionIdTag_>;
using SessionLoginDate = LibPg::Util::StrongType<boost::posix_time::ptime, struct SessionLoginDateTag_>;
using SessionLogoutDate = LibPg::Util::StrongType<boost::posix_time::ptime, struct SessionLogoutDateTag_>;
using UserName = LibPg::Util::StrongType<std::string, struct UserNameTag_>;
using UserId = LibPg::Util::StrongType<unsigned long long, struct UserIdTag_>;

using ServiceId = LibPg::Util::StrongType<int, struct ServiceIdTag_>;
using ServiceName = LibPg::Util::StrongType<std::string, struct ServiceNameTag_>;
using ServiceHandle = LibPg::Util::StrongType<std::string, struct ServiceHandleTag_>;
using IsMonitoring = LibPg::Util::StrongType<bool, struct IsMonitoringTag_>;
using LogEntryId = LibPg::Util::StrongType<unsigned long long, struct LogEntryIdTag_>;
using Content = LibPg::Util::StrongType<std::string, struct ContentTag_>;
using RawRequest = LibPg::Util::StrongType<std::string, struct RawRequestTag_>;
using RawResponse = LibPg::Util::StrongType<std::string, struct RawResponseTag_>;
using IsResponse = LibPg::Util::StrongType<bool, struct IsResponseTag_>;

using PropertyId = LibPg::Util::StrongType<unsigned long long, struct PropertyIdTag_>;
using PropertyNameId = LibPg::Util::StrongType<int, struct PropertyNameIdTag_>;
using PropertyType = LibPg::Util::StrongType<std::string, struct PropertyTypeTag_>;
using PropertyValue = LibPg::Util::StrongType<std::string, struct PropertyValueTag_>;
using IsOutput = LibPg::Util::StrongType<bool, struct IsOutputTag_>;

using LogEntryObjectTypeName = LibPg::Util::StrongType<std::string, struct LogEntryObjectTypeNameTag_>;
using LogEntryObjectTypeId = LibPg::Util::StrongType<unsigned int, struct LogEntryObjectTypeIdTag_>;
using ObjectId = LibPg::Util::StrongType<int, struct ObjectIdTag_>;
using ObjectIdent = LibPg::Util::StrongType<std::string, struct ObjectIdentTag_>;

using LogEntryTypeName = LibPg::Util::StrongType<std::string, struct LogEntryTypeNameTag_>;
using LogEntryTypeId = LibPg::Util::StrongType<unsigned int, struct LogEntryTypeIdTag_>;

using CorbaImplCompatibility = LibPg::Util::StrongType<bool, struct CorbaImplCompatibilityTag_>;

} // namespace Fred::LibLogger::LogEntry

struct LogEntryPropertyValue
{
    LogEntryPropertyValue() = default;

    LogEntryPropertyValue(
            LogEntry::PropertyValue _property_value,
            std::map<LogEntry::PropertyType, std::vector<LogEntry::PropertyValue>> _children);

    LogEntry::PropertyValue property_value;
    std::map<LogEntry::PropertyType, std::vector<LogEntry::PropertyValue>> children;
};

struct LogEntryProperty
{
    LogEntryProperty() = default;

    LogEntryProperty(
            LogEntry::PropertyType _property_type,
            std::vector<LogEntryPropertyValue> _property_values);

    LogEntry::PropertyType property_type;
    std::vector<LogEntryPropertyValue> property_values;
};

struct ObjectReferences
{
    ObjectReferences() = default;

    ObjectReferences(
            LogEntry::LogEntryObjectTypeName _object_type,
            std::vector<LogEntry::ObjectIdent> _object_reference_ids);

    LogEntry::LogEntryObjectTypeName object_type;
    std::vector<LogEntry::ObjectIdent> object_reference_ids;
};

struct WrapperError : std::exception { };
struct UnwrapperError : std::exception { };

} // namespace Fred::LibLogger
} // namespace Fred

#endif
