/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "include/liblogger/log_entry_ident.hh"

#include "include/liblogger/exceptions.hh"

namespace Fred {
namespace LibLogger {

namespace {

LogEntry::LogEntryId validate(const LogEntry::LogEntryId& _id)
{
    if (_id.get() < 1)
    {
        throw InvalidLogEntryIdent{};
    }
    return _id;
}

LogEntry::TimeBegin validate(const LogEntry::TimeBegin& _time_begin)
{
    if (_time_begin.get().is_special())
    {
        throw InvalidLogEntryIdent{};
    }
    return _time_begin;
}

LogEntry::ServiceName validate(const LogEntry::ServiceName& _service_name)
{
    if (_service_name.get().empty())
    {
        throw InvalidLogEntryIdent{};
    }
    return _service_name;
}

} // namespace Fred::LibLogger::{anonymous}

LogEntryIdent::LogEntryIdent(
        const LogEntry::LogEntryId& _id,
        const LogEntry::TimeBegin& _time_begin,
        const LogEntry::ServiceName& _service_handle,
        const LogEntry::IsMonitoring& _monitoring_status)
    : id(validate(_id)),
      time_begin(validate(_time_begin)),
      service_handle(validate(_service_handle)),
      monitoring_status(_monitoring_status)
{
}

} // namespace Fred::LibLogger
} // namespace Fred
