/*
 * Copyright (C) 2019  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef DSN_HH_CF20D56C0327ACE9FDE397F337629849//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define DSN_HH_CF20D56C0327ACE9FDE397F337629849

#include "libpg/util/strong_type.hh"

#include <boost/optional.hpp>
#include <boost/asio/ip/address.hpp>

#include <chrono>
#include <cstdint>
#include <string>

namespace LibPg {

// DSN = Data Source Name
struct Dsn
{
    using Host = Util::StrongType<std::string, struct DsnHostTag_>;
    using HostAddr = Util::StrongType<boost::asio::ip::address, struct DsnHostAddrTag_>;
    using Port = Util::StrongType<std::uint16_t, struct DsnPortTag_>;
    using DbName = Util::StrongType<std::string, struct DsnDbNameTag_>;
    using User = Util::StrongType<std::string, struct DsnUserTag_>;
    using Password = Util::StrongType<std::string, struct DsnPasswordTag_>;
    using ConnectTimeout = Util::StrongType<std::chrono::seconds, struct DsnConnectTimeoutTag_>;

    boost::optional<Host> host;
    boost::optional<HostAddr> host_addr;
    boost::optional<Port> port;
    boost::optional<DbName> db_name;
    boost::optional<User> user;
    boost::optional<Password> password;
    boost::optional<ConnectTimeout> connect_timeout;
};

}//namespace LibPg

#endif//DSN_HH_CF20D56C0327ACE9FDE397F337629849
