/*
 * Copyright (C) 2019  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef STRONG_TYPE_HH_2E688E9B121FE842E5AE1BEA14EC2E97//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define STRONG_TYPE_HH_2E688E9B121FE842E5AE1BEA14EC2E97

#include <type_traits>
#include <utility>

namespace LibPg {
namespace Util {

template <typename Type, typename Tag>
class StrongType
{
public:
    using UnderlyingType = Type;
    template <typename T = Type, std::enable_if_t<std::is_default_constructible<T>::value>* = nullptr>
    constexpr StrongType()
        : value_{}
    { }
    explicit constexpr StrongType(UnderlyingType src)
        : value_{std::move(src)}
    { }
    constexpr const UnderlyingType& get()const&
    {
        return value_;
    }
    constexpr UnderlyingType&& get()&&
    {
        return std::move(value_);
    }
    template <typename F>
    constexpr StrongType& invoke(F&& fnc)
    {
        std::forward<F>(fnc)(value_);
        return *this;
    }
private:
    UnderlyingType value_;
};

}//namespace LibPg::Util
}//namespace LibPg

#endif//STRONG_TYPE_HH_2E688E9B121FE842E5AE1BEA14EC2E97
