/*
 * Copyright (C) 2019  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef STRONG_TYPE_ARITHMETIC_OPERATORS_HH_5D4D71A65F9F8840CE1270441A16AD31//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define STRONG_TYPE_ARITHMETIC_OPERATORS_HH_5D4D71A65F9F8840CE1270441A16AD31

#include "libpg/util/strong_type.hh"
#include "libpg/util/strong_type_comparison_operators.hh"

#include <type_traits>

namespace LibPg {
namespace Util {

template <typename Type, typename Tag,
          std::enable_if_t<sizeof(++std::declval<Type&>()) != 0>* = nullptr>
constexpr StrongType<Type, Tag>& operator++(StrongType<Type, Tag>& value)
{
    struct PreIncrement
    {
        constexpr void operator()(Type& value)const { ++value; }
    };
    value.invoke(PreIncrement{});
    return value;
}

template <typename Type, typename Tag,
          std::enable_if_t<sizeof(std::declval<Type&>()++) != 0>* = nullptr>
constexpr StrongType<Type, Tag> operator++(StrongType<Type, Tag>& value, int)
{
    StrongType<Type, Tag> result{value};
    struct PostIncrement
    {
        constexpr void operator()(Type& value)const { value++; }
    };
    value.invoke(PostIncrement{});
    return result;
}

template <typename Type, typename Tag,
          std::enable_if_t<sizeof(--std::declval<Type&>()) != 0>* = nullptr>
constexpr StrongType<Type, Tag>& operator--(StrongType<Type, Tag>& value)
{
    struct PreDecrement
    {
        constexpr void operator()(Type& value)const { --value; }
    };
    value.invoke(PreDecrement{});
    return value;
}

template <typename Type, typename Tag,
          std::enable_if_t<sizeof(std::declval<Type&>()--) != 0>* = nullptr>
constexpr StrongType<Type, Tag> operator--(StrongType<Type, Tag>& value, int)
{
    StrongType<Type, Tag> result{value};
    struct PostDecrement
    {
        constexpr void operator()(Type& value)const { value--; }
    };
    value.invoke(PostDecrement{});
    return result;
}

template <typename Type, typename Tag,
          std::enable_if_t<std::is_same<decltype(-std::declval<Type>()), Type>::value>* = nullptr>
constexpr StrongType<Type, Tag> operator-(const StrongType<Type, Tag>& a)
{
    return StrongType<Type, Tag>{-a.get()};
}

}//namespace LibPg::Util
}//namespace LibPg

#endif//STRONG_TYPE_ARITHMETIC_OPERATORS_HH_5D4D71A65F9F8840CE1270441A16AD31
